@@ -35,6 +35,7 @@ type StatusReporter interface {
 	LastReportedTime() time.Time
 	SetLastReportedTime(time.Time)
 	SafeInitialStart() bool
+	SetSafeInitialStart(s bool)
 }
 
 type Controller struct {
@@ -136,8 +137,12 @@ func (c *Controller) Run(ctx context.Context) {
 				klog.V(2).Infof("Unable to upload report after %s: %v", time.Now().Sub(start).Truncate(time.Second/100), err)
 				if err == insightsclient.ErrWaitingForVersion {
 					initialDelay = wait.Jitter(interval/8, 1) - interval/8
+					if c.reporter.SafeInitialStart() {
+						initialDelay = wait.Jitter(time.Second*15, 1)
+					}
 					return
 				}
+				c.reporter.SetSafeInitialStart(false)
 				if authorizer.IsAuthorizationError(err) {
 					c.Simple.UpdateStatus(controllerstatus.Summary{Operation: controllerstatus.Uploading,
 						Reason: "NotAuthorized", Message: fmt.Sprintf("Reporting was not allowed: %v", err)})
@@ -150,7 +155,7 @@ func (c *Controller) Run(ctx context.Context) {
 					Reason: "UploadFailed", Message: fmt.Sprintf("Unable to report: %v", err)})
 				return
 			}
-
+			c.reporter.SetSafeInitialStart(false)
 			klog.V(4).Infof("Uploaded report successfully in %s", time.Now().Sub(start))
 			lastReported = start.UTC()
 			c.Simple.UpdateStatus(controllerstatus.Summary{Healthy: true})
